@@ -1,4 +1,5 @@
 # frozen_string_literal: true
+
 require 'molinillo/dependency_graph/action'
 module Molinillo
   class DependencyGraph
@@ -7,8 +8,8 @@ module Molinillo
     class Tag < Action
       # @!group Action
 
-      # (see Action.name)
-      def self.name
+      # (see Action.action_name)
+      def self.action_name
         :tag
       end
 
